import configparser
from pathlib import Path
from typing import Optional

from pydantic import BaseSettings

from .utils import get_current_project_name


class PydevCLISettings(BaseSettings):
    """Configuration for pydev-cli."""

    package_name: Optional[str] = None
    src_dir: Path = Path("src")
    tests_dir: Path = Path("tests")
    dist_dir: Path = Path("dist")
    docs_dir: Path = Path("docs")
    app_import: Optional[str] = None
    docker_image: Optional[str] = None
    docker_tag: str = "latest"
    docker_namespace: str = "gcharbon"


def load_config() -> PydevCLISettings:
    """Load config from setup.cfg or environment variables.

    Returns:
        A PyDevCLISettings instance holding all configuration.
    """
    config = configparser.ConfigParser()
    try:
        config.read("setup.cfg")
        settings = PydevCLISettings(**config["pydev-cli"])
    except (FileNotFoundError, KeyError):
        settings = PydevCLISettings()

    package_name = get_current_project_name()

    if settings.package_name is None:
        settings.package_name = package_name

    if settings.docker_image is None:
        settings.docker_image = settings.package_name

    if settings.app_import is None:
        settings.app_import = f"{settings.package_name}:app"

    return settings



"""This file define scripts that can be used during development."""
import os
from pathlib import Path

import uvicorn
from typer import Argument, Option, Typer, echo

from .config import load_config
from .utils import run


CONFIG = load_config()

SRC_FILES = " ".join([str(path) for path in CONFIG.src_dir.glob("**/*.py")])
TEST_FILES = " ".join([str(path) for path in CONFIG.tests_dir.glob("**/*.py")])


app = Typer(
    name="FastAPI development toolkit",
    no_args_is_help=True,
    help="A command line tool to automate most of development tasks.",
    add_completion=False,
)

services_app = Typer(
    name="Docker services toolkit",
    no_args_is_help=True,
    help="A group of command to manager docker services during development.",
    add_completion=False,
)

app.add_typer(services_app, name="services")


@app.command()
def dev(
    host: str = Option(
        "localhost", envvar="HOST", help="The host your application will listen to.",
    ),
    port: int = Option(
        8000, envvar="PORT", help="The port your application will listen to."
    ),
    reload: bool = Option(
        True,
        envvar="RELOAD",
        help="With hot-reload enabled, your application will restart on each file change.",
    ),
    debug: bool = Option(
        True, envvar="DEBUG", help="Start the application in debug mode."
    ),
) -> None:
    """Start the application in development mode."""
    if debug:
        os.environ["PYTHONASYNCIODEBUG"] = "1"
        os.environ["LOGGING_LEVEL"] = "DEBUG"
    uvicorn.run(CONFIG.app_import, reload=reload, host=host, port=port)


@app.command()
def docs(
    host: str = Option("localhost", help="Host documentation server will listen to."),
    port: int = Option(8080, help="Port the documentation server will listen to."),
) -> None:
    """Serve the documentation on development server."""
    cmd = f"mkdocs serve --dev-addr {host}:{port}"
    run(cmd)


@app.command()
def lint() -> None:
    """Perform linting using flake8."""
    cmd = f"flake8 {SRC_FILES} {TEST_FILES}"
    run(cmd)


@app.command()
def format() -> None:
    """Format the code using black and isort."""
    black_cmd = f"black {SRC_FILES} {TEST_FILES}"
    isort_cmd = f"isort {SRC_FILES} {TEST_FILES}"
    run(black_cmd)
    run(isort_cmd)


@app.command()
def test() -> None:
    """Run the tests using pytest."""
    os.environ["PYTHONASYNCIODEBUG"] = "1"
    os.environ["LOGGING_LEVEL"] = "DEBUG"
    os.environ["LOGGING_TEST_MODE"] = "1"
    cmd = "pytest"
    run(cmd)


@app.command()
def typecheck() -> None:
    """Run type checking using mypy."""
    cmd = f"mypy {CONFIG.src_dir} {CONFIG.tests_dir}"
    run(cmd)


@app.command()
def build(
    package: bool = Option(True, help="Build the python package in wheel format.",),
    docs: bool = Option(True, help="Build the documentation."),
    coverage: bool = Option(
        True, help="Run the unit tests before building documentation or package."
    ),
    verbose: bool = Option(False, help="Show python traceback if any task failed."),
) -> None:
    """Run tests, build documentation and built package."""
    if coverage:
        pytest_cmd = "pytest"
        if not Path(CONFIG.docs_dir / "coverage-report" / "index.html").is_file():
            echo("Running tests to generate coverage report.")
            run(pytest_cmd)
    if docs:
        mkdocs_cmd = f"mkdocs build -d {CONFIG.docs_dir}"
        run(mkdocs_cmd)
    if package:
        poetry_cmd = "poetry build"
        run(poetry_cmd, verbose=verbose)


@app.command()
def publish(repository: str = "pypi") -> None:
    """Publish package to external repository."""
    poetry_cmd = f"poetry publish --repository {repository}"
    run(poetry_cmd)


@app.command()
def build_docker_image(
    namespace: str = CONFIG.docker_namespace,
    image: str = CONFIG.docker_image,  # type: ignore
    tag: str = CONFIG.docker_tag,
) -> None:
    """Build the docker image for production usage."""
    docker_cmd = f"docker build -t {namespace}/{image}:{tag} -f docker/Dockerfile ."
    run(docker_cmd)


@app.command()
def build_ci_docker_image(
    namespace: str = CONFIG.docker_namespace,
    image: str = f"{CONFIG.docker_image}-ci",
    tag: str = "latest",
) -> None:
    """Build the docker image for continuous integration usage."""
    docker_cmd = f"docker build -t {namespace}/{image}:{tag} -f docker/Dockerfile ."
    run(docker_cmd)


@app.command()
def nb_kernel(
    display_name: str = Argument(
        CONFIG.package_name, help="Displayed name for the kernel."
    )
) -> None:
    """Create an IPython kernel to use with jupyter notebook."""
    try:
        import ipykernel as _  # noqa: F401
    except ModuleNotFoundError:
        run("poetry add --dev ipykernel")
    run(
        "python -m ipykernel install --user"
        f' --name "{CONFIG.package_name}"'
        f' --display-name "{display_name}"'
    )


@services_app.command()
def start(local: bool = True, app: bool = False) -> None:
    """Start a service."""
    if local:
        run(
            "docker stack deploy "
            f"{'-c compose/docker-compose.yml ' if app else ''}"
            "-c compose/docker-compose.dependencies.yml "
            "-c compose/docker-compose.local.yml "
            f"local-{CONFIG.package_name}"
        )
    else:
        run(
            "docker stack deploy "
            f"{'-c compose/docker-compose.yml ' if app else ''}"
            "-c compose/docker-compose.dependencies.yml "
            "-c compose/docker-compose.traefik.yml"
            f"dev-{CONFIG.package_name}"
        )


@services_app.command()
def stop(local: bool = True) -> None:
    """Stop a service."""
    if local:
        run(f"docker stack rm local-{CONFIG.package_name}")
    else:
        run(f"docker stack rm dev-{CONFIG.package_name}")



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['pydev_cli']

package_data = \
{'': ['*']}

install_requires = \
['black>=19.10b0,<20.0',
 'flake8>=3.8.3,<4.0.0',
 'isort>=5.5.1,<6.0.0',
 'mypy>=0.790,<0.791',
 'pre-commit>=2.7.1,<3.0.0',
 'pydantic>=1.6.1,<2.0.0',
 'pytest-cov>=2.10.1,<3.0.0',
 'pytest>=6.0.1,<7.0.0',
 'python-semantic-release>=7.3.0,<8.0.0',
 'toml>=0.10.1,<0.11.0',
 'typer>=0.3.2,<0.4.0',
 'uvicorn>=0.12.1,<0.13.0']

entry_points = \
{'console_scripts': ['pydev = pydev_cli.cli:app']}

setup_kwargs = {
    'name': 'pydev-cli',
    'version': '0.2.0',
    'description': 'A single python development dependency to rule them all',
    'long_description': None,
    'author': 'Guillaume Charbonnier',
    'author_email': 'guillaume.charbonnier@capgemini.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



"""Useful functions for pydev-cli package."""
import platform
import sys
from pathlib import Path
from typing import TextIO
from subprocess import check_call

import toml
from typer import echo


IS_WINDOWS = platform.system() == "Windows"


def get_current_project_name() -> str:
    """Find a pyproject.toml file in curretn directory and return project name, else raise a FileNotFoundError.

    Returns:
        The name of the python package in current directory.

    Raises:
        FileNotFoundError: Proces stopped because file "pyproject.toml" could not be found.
    """
    filepath = Path("pyproject.toml")
    if not filepath.is_file():
        raise FileNotFoundError("Could not find 'pyproject.toml' file.")
    content = toml.loads(filepath.read_text())
    return content["tool"]["poetry"]["name"]


def run(
    cmd: str,
    stdout: TextIO = sys.stdout,
    stderr: TextIO = sys.stderr,
    verbose: bool = False,
) -> None:
    """Run a command and forward both stdout and stderr to console by default.

    In case of error, process will exit with status code 1.

    Arguments:
        cmd: The command to execute.
        stdout: A TextIO object where process standard output will be redirected.
        stderr: A TextIO object where process standard error will be redirected.
        verbose: Print error traceback in case of process failure.
    """
    try:
        check_call(cmd, stdout=stdout, stderr=stderr, shell=IS_WINDOWS)
    except Exception as error:
        echo(f"Task failed on command: {cmd}", file=sys.stderr)
        if verbose:
            echo(f"{error}", file=sys.stderr)
        sys.exit(1)
